from django.test import TestCase

from geokey.categories.tests.model_factories import CategoryFactory

from ..templatetags import tags


class TemplateTagsTest(TestCase):
    def test_show_restrict(self):
        category = CategoryFactory.create()
        self.assertEqual(
            tags.show_restrict({str(category.id): {}}, category),
            '<a href="#" class="text-danger activate-detailed">'
            'Restrict further</a>'
        )
        self.assertEqual(
            tags.show_restrict({'2': {}}, category),
            ''
        )

    def test_is_selected(self):
        dict = ["1", "2", "3"]

        self.assertEqual(tags.is_selected(1, dict), 'selected')
        self.assertEqual(tags.is_selected(4, dict), '')

    def test_is_in(self):
        dict = {
            '1': {},
            '2': {}
        }

        self.assertTrue(tags.is_in(dict, 1))
        self.assertFalse(tags.is_in(dict, 4))

    def test_minval(self):
        self.assertEqual(tags.minval({'minval': 5}), 5)
        self.assertEqual(tags.minval({}), '')

    def test_maxval(self):
        self.assertEqual(tags.maxval({'maxval': 5}), 5)
        self.assertEqual(tags.maxval({}), '')



import json
from rest_framework.parsers import BaseParser


class GeoJsonParser(BaseParser):
    """Parses GeoJson into deserialisable native Python objects """
    media_type = 'application/json'

    def parse(self, stream, media_type=None, parser_context=None):
        """
        Parses the GeoJson imput and returns deserialisable Python native
        """
        parser_context = parser_context or {}

        request_data = stream.read()
        data = json.loads(request_data)

        if 'geometry' in data:
            if 'location' not in data:
                data['location'] = {}

            data['location']['geometry'] = json.dumps(data.pop('geometry'))

        return data



from django.core.exceptions import PermissionDenied
from django.test import TestCase

from nose.tools import raises

from geokey.projects.tests.model_factories import ProjectF, UserF

from ..model_factories import LocationFactory
from geokey.contributions.models import Location


class LocationTest(TestCase):
    def setUp(self):
        self.admin = UserF.create()

        self.project1 = ProjectF(add_admins=[self.admin])
        self.project2 = ProjectF(add_admins=[self.admin])
        self.public_location = LocationFactory(**{
            'private': False
        })
        self.private_location = LocationFactory(**{
            'private': True
        })
        self.private_for_project_location = LocationFactory(**{
            'private': True,
            'private_for_project': self.project1
        })

    def test_get_locations_for_project1_with_admin(self):
        locations = Location.objects.get_list(self.admin, self.project1.id)
        self.assertEqual(len(locations), 2)

    def test_get_locations_for_project2_with_admin(self):
        locations = Location.objects.get_list(self.admin, self.project2.id)
        self.assertEqual(len(locations), 1)

    def test_get_public_location_for_project1_with_admin(self):
        location = Location.objects.get_single(
            self.admin, self.project1.id, self.public_location.id)
        self.assertEqual(location, self.public_location)

    def test_get_project_location_for_project1_with_admin(self):
        location = Location.objects.get_single(
            self.admin, self.project1.id, self.private_for_project_location.id)
        self.assertEqual(location, self.private_for_project_location)

    @raises(PermissionDenied)
    def test_get_private_location_for_project1_with_admin(self):
        Location.objects.get_single(
            self.admin, self.project1.id, self.private_location.id)

    def test_get_public_location_for_project2_with_admin(self):
        location = Location.objects.get_single(
            self.admin, self.project2.id, self.public_location.id)
        self.assertEqual(location, self.public_location)

    @raises(PermissionDenied)
    def test_get_project_location_for_project2_with_admin(self):
        Location.objects.get_single(
            self.admin, self.project2.id, self.private_for_project_location.id)

    @raises(PermissionDenied)
    def test_get_private_location_for_project2_with_admin(self):
        Location.objects.get_single(
            self.admin, self.project2.id, self.private_location.id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(null=True, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'inactive', b'inactive'), (b'deleted', b'deleted')])),
                ('default_status', models.CharField(default=b'pending', max_length=20, choices=[(b'active', b'active'), (b'pending', b'pending')])),
                ('colour', models.TextField(default=b'#0033ff')),
                ('symbol', models.ImageField(null=True, upload_to=b'symbols')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Field',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('key', models.CharField(max_length=103)),
                ('description', models.TextField(null=True, blank=True)),
                ('required', models.BooleanField(default=False)),
                ('order', models.IntegerField(default=0)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'inactive', b'inactive'), (b'deleted', b'deleted')])),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DateTimeField',
            fields=[
                ('field_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='categories.Field')),
            ],
            options={
            },
            bases=('categories.field',),
        ),
        migrations.CreateModel(
            name='LookupField',
            fields=[
                ('field_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='categories.Field')),
            ],
            options={
            },
            bases=('categories.field',),
        ),
        migrations.CreateModel(
            name='LookupValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'inactive', b'inactive'), (b'deleted', b'deleted')])),
                ('field', models.ForeignKey(related_name='lookupvalues', to='categories.LookupField')),
            ],
            options={
                'ordering': ['id'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MultipleLookupField',
            fields=[
                ('field_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='categories.Field')),
            ],
            options={
            },
            bases=('categories.field',),
        ),
        migrations.CreateModel(
            name='MultipleLookupValue',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'inactive', b'inactive'), (b'deleted', b'deleted')])),
                ('field', models.ForeignKey(related_name='lookupvalues', to='categories.MultipleLookupField')),
            ],
            options={
                'ordering': ['id'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='NumericField',
            fields=[
                ('field_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='categories.Field')),
                ('minval', models.FloatField(null=True, blank=True)),
                ('maxval', models.FloatField(null=True, blank=True)),
            ],
            options={
            },
            bases=('categories.field',),
        ),
        migrations.CreateModel(
            name='TextField',
            fields=[
                ('field_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='categories.Field')),
            ],
            options={
            },
            bases=('categories.field',),
        ),
        migrations.AddField(
            model_name='field',
            name='category',
            field=models.ForeignKey(related_name='fields', to='categories.Category'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='field',
            unique_together=set([('key', 'category')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RunSQL('DROP TABLE IF EXISTS applications_application CASCADE;'),
        migrations.RunSQL("DELETE FROM django_migrations WHERE app = 'applications';")
    ]



import os.path
from geokey.core.settings.dev import *

# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases
DEFAULT_FROM_EMAIL = 'sender@example.com'
ACCOUNT_EMAIL_VERIFICATION = 'optional'

DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'geokey',                      # Or path to database file if using sqlite3.
        # The following settings are not used with sqlite3:
        'USER': 'django',
        'PASSWORD': 'django123',
        'HOST': 'localhost',                      # Empty for localhost through domain sockets or '127.0.0.1' for localhost through TCP.
        'PORT': '',                      # Set to empty string for default.
    }
}

SECRET_KEY = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'

ENABLE_VIDEO = True
YOUTUBE_AUTH_EMAIL = 'your-email@example.com'
YOUTUBE_AUTH_PASSWORD = 'password'
YOUTUBE_DEVELOPER_KEY = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'
YOUTUBE_CLIENT_ID = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx.apps.googleusercontent.com'

INSTALLED_APPS += (
    'geokey_epicollect',
    # 'geokey_cartodb',
    'geokey_communitymaps',
    'geokey_sapelli',
    'geokey_export',
    'geokey_geotagx',
    'pantechnicon',
)

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'

MEDIA_ROOT = normpath(join(dirname(dirname(abspath(__file__))), 'assets'))
MEDIA_URL = '/assets/'

WSGI_APPLICATION = 'local_settings.wsgi.application'

if os.path.exists('local_settings/urls.py'):
    ROOT_URLCONF = 'local_settings.urls'
else:
    ROOT_URLCONF = 'geokey.core.urls'

def show_toolbar(request):
    print 'blah'
    return True
SHOW_TOOLBAR_CALLBACK = show_toolbar

# LOGGING = {
#     'version': 1,
#     'disable_existing_loggers': True,
#     'formatters': {
#         'verbose': {
#             'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
#         },
#     },
#     'handlers': {
#         'console': {
#             'level': 'NOTSET',
#             'class': 'logging.StreamHandler',
#             'formatter': 'verbose'
#         }
#     },
#     'loggers': {
#         '': {
#             'handlers': ['console'],
#             'level': 'NOTSET',
#         },
#         'django.request': {
#             'handlers': ['console'],
#             'propagate': False,
#             'level': 'ERROR'
#         }
#     }
# }



from django.test import TestCase
from django.core.exceptions import PermissionDenied

from nose.tools import raises

from geokey.projects.tests.model_factories import UserF

from .model_factories import ApplicationFactory

from ..models import Application


class ApplicationManagerTest(TestCase):
    def setUp(self):
        self.user1 = UserF.create()
        self.user2 = UserF.create()
        self.app1 = ApplicationFactory(**{
            'user': self.user1
        })
        self.app2 = ApplicationFactory(**{
            'user': self.user1
        })
        self.app3 = ApplicationFactory(**{
            'user': self.user2
        })
        self.deleted_app = ApplicationFactory(**{
            'user': self.user1,
            'status': 'deleted'
        })

    def test_get_apps_with_user1(self):
        apps = Application.objects.get_list(self.user1)
        self.assertEqual(len(apps), 2)
        self.assertNotIn(self.deleted_app, apps)
        self.assertNotIn(self.app3, apps)

    def test_get_apps_with_user2(self):
        apps = Application.objects.get_list(self.user2)
        self.assertEqual(len(apps), 1)
        self.assertNotIn(self.deleted_app, apps)
        self.assertNotIn(self.app1, apps)
        self.assertNotIn(self.app2, apps)

    def test_get_single_app_with_user1(self):
        app = Application.objects.as_owner(self.user1, self.app1.id)
        self.assertEqual(app, self.app1)

    @raises(PermissionDenied)
    def test_get_single_app_with_user2(self):
        Application.objects.as_owner(self.user2, self.app1.id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('applications', '0002_auto_20150112_1700'),
    ]

    operations = [
        migrations.AddField(
            model_name='application',
            name='skip_authorization',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='application',
            name='user',
            field=models.ForeignKey(related_name='applications_application', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
    ]



import factory
from PIL import Image
from StringIO import StringIO

from django.core.files.base import ContentFile

from geokey.users.tests.model_factories import UserF
from geokey.contributions.tests.model_factories import ObservationFactory
from geokey.contributions.models import ImageFile, VideoFile


def get_image(file_name='test.png', width=200, height=200):
    image_file = StringIO()
    image = Image.new('RGBA', size=(width, height), color=(255, 0, 255))
    image.save(image_file, 'png')
    image_file.seek(0)

    the_file = ContentFile(image_file.read(), file_name)
    the_file.content_type = 'image/png'

    return the_file


class ImageFileFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ImageFile

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    creator = factory.SubFactory(UserF)
    contribution = factory.SubFactory(ObservationFactory)
    image = get_image()


class VideoFileFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = VideoFile

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    creator = factory.SubFactory(UserF)
    contribution = factory.SubFactory(ObservationFactory)
    youtube_id = 'qb2Cd1--YvY'
    youtube_link = 'http://example.com/98aishfijh'
    swf_link = 'http://example.com/98aishfijh'



from django.views.generic import TemplateView
from django.contrib.sites.shortcuts import get_current_site
from django.contrib import messages

from braces.views import LoginRequiredMixin

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import BasePermission
from rest_framework import status

from geokey.projects.models import Project
from geokey.users.models import User
from geokey.users.serializers import UserSerializer


# #############################################################################
#
# ADMIN VIEWS
#
# #############################################################################


class SuperuserMixin(object):
    """
    Mixin to check if the user is a super user
    """
    def dispatch(self, request, *args, **kwargs):
        """
        Dispatches the request. Either responds with an Error message or calls
        View's dispatch method.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.

        Returns
        -------
        django.http.HttpResponse

        """
        if not request.user.is_superuser:
            return self.render_to_response({
                'error_description': 'Superuser tools are for superusers only.'
                                     ' You are not a superuser.',
                'error': 'Permission denied.'
            })

        return super(SuperuserMixin, self).dispatch(request, *args, **kwargs)


class PlatformSettings(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """
    Change the settings of the platform
    """
    template_name = 'superusertools/platform_settings.html'

    def get_context_data(self):
        """
        Returns the context to render the view, adds current site to context

        Returns
        -------
        dict
        """
        return {'site': get_current_site(self.request)}

    def post(self, request):
        """
        Updates the platform settings

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.

        Returns
        -------
        django.http.HttpResponse
        """
        data = request.POST
        context = self.get_context_data()
        site = context.pop('site', None)

        if site is not None:
            site.name = data.get('name')
            site.domain = data.get('domain')
            site.save()
            messages.success(
                self.request,
                "The platform settings have been updated."
            )

        context['site'] = site
        return self.render_to_response(context)


class ProjectsList(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """
    Displays a list of all projects
    """
    template_name = 'superusertools/projects_list.html'

    def get_context_data(self):
        """
        Returns the context to render the view, adds list of projects to
        context

        Returns
        -------
        dict
        """
        return {'projects': Project.objects.all()}


class ManageSuperUsers(LoginRequiredMixin, SuperuserMixin, TemplateView):
    """
    View to add and remove superusers
    """
    template_name = 'superusertools/manage_users.html'

    def get_context_data(self):
        """
        Returns the context to render the view, adds list of superusers to
        context

        Returns
        -------
        dict
        """
        return {'superusers': User.objects.filter(is_superuser=True)}


# #############################################################################
#
# ADMIN AJAX VIEWS
#
# #############################################################################


class IsSuperUser(BasePermission):
    """
    Checks whether the authenticated user is a superiser
    """
    def has_permission(self, request, view):
        """
        Returns True if the user is a superuser

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request
        view : rest_framework.views.APIView
            View that called the permission

        Return
        ------
        Boolean
            indicating if user is a super user
        """
        return request.user and request.user.is_superuser


class AddSuperUsersAjaxView(APIView):
    """
    AJAX API endpoint to add superusers
    """
    permission_classes = (IsSuperUser,)

    def post(self, request):
        """
        Adds a new superuser

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request

        Returns
        -------
        rest_framework.response.Response
            Contains the list of superusers or an error message
        """
        try:
            user = User.objects.get(pk=request.DATA.get('userId'))
            user.is_superuser = True
            user.save()

            superusers = User.objects.filter(is_superuser=True)
            serializer = UserSerializer(superusers, many=True)
            return Response(
                {'users': serializer.data},
                status=status.HTTP_201_CREATED
            )

        except User.DoesNotExist:
            return Response(
                'The user you are trying to add to the user group does ' +
                'not exist',
                status=status.HTTP_400_BAD_REQUEST
            )


class DeleteSuperUsersAjaxView(APIView):
    """
    AJAX API endpoint to remove superusers
    """
    permission_classes = (IsSuperUser,)

    def delete(self, request, user_id):
        """
        Deletes a superuser

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request

        Returns
        -------
        rest_framework.response.Response
            Empty response indicating success an error message
        """
        try:
            user = User.objects.filter(is_superuser=True).get(pk=user_id)
            user.is_superuser = False
            user.save()

            return Response(status=status.HTTP_204_NO_CONTENT)

        except User.DoesNotExist:
            return Response(
                'Superuser does not exist.',
                status=status.HTTP_404_NOT_FOUND
            )



from django.core.exceptions import PermissionDenied

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from geokey.core.exceptions import MalformedRequestData
from geokey.core.decorators import handle_exceptions_for_ajax
from .base import SingleAllContribution

from geokey.users.models import User
from ..serializers import FileSerializer
from ..models import MediaFile


class MediaFileListAbstractAPIView(APIView):
    """
    Abstract APIView for listing media files of a contribution
    """
    def get_list_and_respond(self, user, contribution):
        """
        Serialises all files attached to the contribution and returns the
        JSON response

        Parameter
        ---------
        user : geokey.users.models.User
            User authenticated with the request
        contribution : geokey.contributions.models.Observation
            contribution the comments are requested for

        Returns
        -------
        rest_framework.response.Respones
            Contains the serialised files
        """
        files = contribution.files_attached.all()
        serializer = FileSerializer(
            files, many=True, context={'user': user})
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create_and_respond(self, user, contribution):
        """
        Creates an file and responds with the file information

        Parameter
        ---------
        user : geokey.users.models.User
            User authenticated with the request
        contribution : geokey.contributions.models.Observation
            contribution the comments are requested for

        Returns
        -------
        rest_framework.response.Respones
            Contains the serialised file
        """
        if user.is_anonymous():
            user = User.objects.get(display_name='AnonymousUser')

        data = self.request.POST
        name = data.get('name')
        description = data.get('description')
        the_file = self.request.FILES.get('file')
        errors = []

        if name is None:
            errors.append('Property name is not set.')

        if the_file is None:
            errors.append('No file attached.')

        if errors:
            raise MalformedRequestData(', '.join(errors))

        if contribution.project.can_contribute(user):
            the_file = MediaFile.objects.create(
                name=name,
                description=description,
                contribution=contribution,
                creator=user,
                the_file=the_file
            )

            serializer = FileSerializer(the_file, context={'user': user})
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        else:
            raise PermissionDenied('You are not allowed to contribute to the'
                                   'project.')


class AllContributionsMediaAPIView(
        MediaFileListAbstractAPIView, SingleAllContribution):
    """
    View for media file lists on all contributions endpoints
    """

    @handle_exceptions_for_ajax
    def get(self, request, project_id, contribution_id):
        """
        Returns a list of all files attached to the observation

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base
        contribution_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comments
        """
        contribution = self.get_object(
            request.user,
            project_id,
            contribution_id
        )
        return self.get_list_and_respond(request.user, contribution)

    @handle_exceptions_for_ajax
    def post(self, request, project_id, contribution_id):
        """
        Creates a new file for a contribution

        Parameters
        ----------
        request : rest_framework.request.Request
            Represents the request
        project_id : int
            identifies the project in the data base
        contribution_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comments
        """
        contribution = self.get_object(
            request.user,
            project_id,
            contribution_id
        )
        return self.create_and_respond(request.user, contribution)


class MediaFileSingleAbstractView(APIView):
    """
    Abstract APIView for single file endpoints.
    """
    def get_and_respond(self, user, media_file):
        """
        Returns a single media file

        Parameter
        ---------
        user : geokey.users.models.User
            User authenticated with the request
        media_file : geokey.contributions.models.MediaFile
            File to be returned

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised file
        """
        serializer = FileSerializer(media_file, context={'user': user})
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete_and_respond(self, user, media_file):
        """
        Deletes a single media file

        Parameter
        ---------
        user : geokey.users.models.User
            User authenticated with the request
        media_file : geokey.contributions.models.MediaFile
            File to be deleted

        Returns
        -------
        rest_framework.response.Respone
            Empty response stating the success of the delete
        """
        if (media_file.creator == user or
                media_file.contribution.project.can_moderate(user)):
            media_file.delete()
            return Response(status=status.HTTP_204_NO_CONTENT)

        raise PermissionDenied('You neither are the creator if this file '
                               'nor a project moderator and therefore '
                               'not eligable to delete this file.')


class AllContributionsSingleMediaApiView(
        MediaFileSingleAbstractView, SingleAllContribution):
    """
    APIView for single files on all contributions endpoints
    """

    def get_file(self, user, project_id, contribution_id, file_id):
        """
        Returns the file object

        Parameter
        ---------
        user : geokey.users.models.User
            User authenticated with the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base
        file_id : int
            identifies the file in the database

        Returns
        -------
        geokey.contributions.models.MediaFile
        """
        contribution = self.get_object(user, project_id, contribution_id)
        return contribution.files_attached.get(pk=file_id)

    @handle_exceptions_for_ajax
    def get(self, request, project_id, contribution_id, file_id):
        """
        Returns a single media file

        Parameter
        ---------
        user : geokey.users.models.User
            User authenticated with the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base
        file_id : int
            identifies the file in the database

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised file
        """
        media_file = self.get_file(
            request.user,
            project_id,
            contribution_id,
            file_id
        )
        return self.get_and_respond(request.user, media_file)

    @handle_exceptions_for_ajax
    def delete(self, request, project_id, contribution_id, file_id):
        """
        Deletes a single media file

        Parameter
        ---------
        user : geokey.users.models.User
            User authenticated with the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base
        file_id : int
            identifies the file in the database

        Returns
        -------
        rest_framework.response.Respone
            Empty repsonse indicating success
        """
        media_file = self.get_file(
            request.user,
            project_id,
            contribution_id,
            file_id
        )
        return self.delete_and_respond(request.user, media_file)



from django.test import TestCase

from ..model_factories import ObservationFactory, CommentFactory
from geokey.contributions.models import Comment


class CommentTest(TestCase):
    def test_get_comments(self):
        observation = ObservationFactory.create()
        CommentFactory.create_batch(5, **{'commentto': observation})
        CommentFactory.create(**{
            'commentto': observation,
            'status': 'deleted'
        })
        comments = Comment.objects.all()
        self.assertEqual(len(comments), 5)
        for comment in comments:
            self.assertNotEqual(comment.status, 'deleted')



from django.db import models
from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser
from django.utils import timezone

from django_pgjson.fields import JsonBField
from oauth2_provider.models import AccessToken

from .manager import UserManager


class User(AbstractBaseUser):
    """
    A user registered in the platform.
    """
    email = models.EmailField(unique=True)
    display_name = models.CharField(
        max_length=50,
        unique=True
    )
    date_joined = models.DateTimeField(default=timezone.now)
    is_active = models.BooleanField(default=True)
    is_superuser = models.BooleanField(default=False)

    USERNAME_FIELD = 'display_name'
    REQUIRED_FIELDS = ['email']

    objects = UserManager()

    def reset_password(self, password):
        """
        Resets the users password and deletes all access tokens assigned to the
        user.

        Parameters
        ----------
        password : str
            New password of the user
        """
        self.set_password(password)
        self.save()
        AccessToken.objects.filter(user=self).delete()


class UserGroup(models.Model):
    """
    A user gropup assigned to a project.
    """
    name = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    users = models.ManyToManyField(settings.AUTH_USER_MODEL)
    project = models.ForeignKey('projects.Project', related_name='usergroups')
    can_contribute = models.BooleanField(default=True)
    can_moderate = models.BooleanField(default=False)
    filters = JsonBField(blank=True, null=True)
    where_clause = models.TextField(blank=True, null=True)

    def save(self, *args, **kwargs):
        """
        Overwrites save to implement integrity ensurance.
        """
        self.where_clause = None
        if self.filters is not None:
            queries = []

            for key in self.filters:
                category = self.project.categories.get(pk=key)
                queries.append(category.get_query(self.filters[key]))

            if len(queries) > 0:
                query = ' OR '.join(queries)
                self.where_clause = query
            else:
                self.where_clause = 'FALSE'

        if self.can_moderate:
            self.can_contribute = True

        super(UserGroup, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-

from django.test import TestCase
from django.core.exceptions import ValidationError

from nose.tools import raises

from geokey.contributions.models import Observation, pre_save_update
from geokey.projects.tests.model_factories import UserF

from geokey.categories.tests.model_factories import (
    CategoryFactory, LookupFieldFactory, LookupValueFactory,
    TextFieldFactory, MultipleLookupFieldFactory, MultipleLookupValueFactory,
    NumericFieldFactory
)
from ..model_factories import ObservationFactory, LocationFactory


class TestContributionsPreSave(TestCase):
    def test_pre_save(self):
        o_type = CategoryFactory.create()
        TextFieldFactory.create(
            **{'key': 'key', 'category': o_type, 'order': 0}
        )

        lookup = LookupFieldFactory.create(
            **{'category': o_type, 'key': 'lookup', 'order': 1}
        )
        kermit = LookupValueFactory.create(**{
            'field': lookup,
            'name': 'Kermit'
        })
        LookupValueFactory.create(**{
            'field': lookup,
            'name': 'Gonzo'
        })

        m_lookup = MultipleLookupFieldFactory.create(
            **{'category': o_type, 'key': 'm_lookup', 'order': 2}
        )
        m_kermit = MultipleLookupValueFactory.create(**{
            'field': m_lookup,
            'name': 'Kermit'
        })
        MultipleLookupValueFactory.create(**{
            'field': m_lookup,
            'name': 'Gonzo'
        })
        m_piggy = MultipleLookupValueFactory.create(**{
            'field': m_lookup,
            'name': 'Ms Piggy'
        })

        o = ObservationFactory.create(**{
            'properties': {
                'key': 'blah',
                'lookup': kermit.id,
                'm_lookup': [m_kermit.id, m_piggy.id]
            },
            'category': o_type
        })

        pre_save_update(Observation, instance=o)
        self.assertIn('Ms Piggy', o.search_matches)
        self.assertIn('Kermit', o.search_matches)
        self.assertIn('blah', o.search_matches)


class ObservationTest(TestCase):
    @raises(Observation.DoesNotExist)
    def test_delete_observation(self):
        observation = ObservationFactory()
        observation.delete()
        Observation.objects.get(pk=observation.id)

    def test_update_display_field(self):
        category = CategoryFactory()
        field = TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        category.display_field = field
        category.save()

        observation = ObservationFactory(**{
            'project': category.project,
            'category': category,
            'display_field': None,
            'properties': {
                'text': 'blah'
            }
        })

        observation.update_display_field()
        observation.save()

        ref = Observation.objects.get(pk=observation.id)
        self.assertEqual(ref.display_field, 'text:blah')

    def test_update_display_field_empty_properties(self):
        category = CategoryFactory()
        field = TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        category.display_field = field
        category.save()

        observation = ObservationFactory(**{
            'project': category.project,
            'category': category,
            'display_field': None,
            'properties': None
        })

        observation.update_display_field()
        observation.save()

        ref = Observation.objects.get(pk=observation.id)
        self.assertEqual(ref.display_field, 'text:None')

    def test_update_display_field_no_display_field(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        observation = ObservationFactory(**{
            'project': category.project,
            'category': category,
            'display_field': None,
            'properties': None
        })

        observation.update_display_field()
        observation.save()

        ref = Observation.objects.get(pk=observation.id)
        self.assertIsNone(ref.display_field)

    def test_create_observation(self):
        creator = UserF()
        location = LocationFactory()
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'text': 'Text', 'number': 12}
        observation = Observation.create(
            properties=data, creator=creator, location=location,
            category=category, project=category.project, status='active'
        )
        self.assertEqual(observation.properties, data)

    def test_create_observation_with_polish_chars(self):
        creator = UserF()
        location = LocationFactory()
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'required': True,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'text': u'śmietnik', 'number': 12}
        observation = Observation.create(
            properties=data, creator=creator, location=location,
            category=category, project=category.project, status='active'
        )
        self.assertEqual(observation.properties, data)

    def test_create_observation_active_default(self):
        creator = UserF()
        location = LocationFactory()
        category = CategoryFactory(**{
            'default_status': 'active'
        })
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'text': 'Text', 'number': 12}
        observation = Observation.create(
            properties=data, creator=creator, location=location,
            category=category, project=category.project, status='active'
        )
        self.assertEqual(observation.properties, data)

    def test_validate_full_inactive_field(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 2
        })
        TextFieldFactory(**{
            'key': 'inactive_text',
            'category': category,
            'status': 'inactive',
            'required': True,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'text': 'Text', 'number': 12}
        Observation.validate_full(category=category, data=data)

    def test_update_observation(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })

        observation = ObservationFactory.create(**{
            'properties': {'text': 'Text', 'number': 12},
            'category': category,
            'project': category.project
        })

        updater = UserF()
        update = {'text': 'Udpated Text', 'number': 13}
        observation.update(properties=update, updator=updater)

        # ref_observation = Observation.objects.get(pk=observation.id)
        self.assertEqual(
            observation.properties,
            {'text': 'Udpated Text', 'number': 13}
        )
        self.assertEqual(observation.version, 2)

    def test_validate_full_with_inactive_field(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        TextFieldFactory(**{
            'key': 'inactive_text',
            'category': category,
            'status': 'inactive',
            'required': True,
            'order': 1
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 2
        })

        observation = ObservationFactory.create(**{
            'properties': {'text': 'Text', 'number': 12},
            'category': category,
            'project': category.project
        })

        updater = UserF()
        update = {'text': 'Udpated Text', 'number': 13}
        Observation.validate_full(category=category, data=update)
        observation.update(properties=update, updator=updater)

        self.assertEqual(
            observation.properties,
            {'text': 'Udpated Text', 'number': 13}
        )
        self.assertEqual(observation.version, 2)

    @raises(ValidationError)
    def test_validate_full_invalid(self):
        creator = UserF()
        location = LocationFactory()
        category = CategoryFactory()
        TextFieldFactory.create(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        NumericFieldFactory.create(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'text': 'Text', 'number': 12}
        observation = Observation.create(
            properties=data, creator=creator, location=location,
            category=category, project=category.project, status='active'
        )

        updater = UserF()
        update = {'text': 'Udpated Text', 'number': 'abc', 'version': 1}
        Observation.validate_full(category=category, data=update)
        observation.update(properties=update, updator=updater)

        self.assertEqual(observation.properties, data)
        self.assertEqual(observation.version, 1)

    @raises(ValidationError)
    def test_validate_full_invalid_nubmer(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'text': 'Text', 'number': 'abc'}
        Observation.validate_full(data=data, category=category)

    @raises(ValidationError)
    def test_validate_full_with_empty_textfield(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'required': True,
            'category': category,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'number': 1000}
        Observation.validate_full(data=data, category=category)

    @raises(ValidationError)
    def test_validate_full_with_zero_textfield(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'required': True,
            'category': category,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'text': '', 'number': 1000}
        Observation.validate_full(data=data, category=category)

    def test_update_draft_observation(self):
        creator = UserF()
        location = LocationFactory()
        category = CategoryFactory()
        TextFieldFactory.create(**{
            'key': 'text',
            'category': category,
            'required': True,
            'order': 0
        })
        NumericFieldFactory.create(**{
            'key': 'number',
            'category': category,
            'order': 1
        })
        data = {'number': 12}
        observation = Observation.create(
            properties=data, creator=creator, location=location,
            category=category, project=category.project,
            status='draft'
        )

        updater = UserF()
        update = {'number': 13}
        observation.update(properties=update, updator=updater, status='draft')

        self.assertEqual(observation.properties.get('number'), 13)
        self.assertEqual(observation.version, 1)

    @raises(ValidationError)
    def test_validate_full_with_empty_number(self):
        category = CategoryFactory()
        TextFieldFactory(**{
            'key': 'text',
            'category': category,
            'order': 0
        })
        NumericFieldFactory(**{
            'key': 'number',
            'required': True,
            'category': category,
            'order': 1
        })
        data = {'text': 'bla'}
        Observation.validate_full(data=data, category=category)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django_hstore.fields
import django.contrib.gis.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('text', models.TextField()),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'deleted', b'deleted')])),
                ('review_status', models.CharField(blank=True, max_length=10, null=True, choices=[(b'open', b'open'), (b'resolved', b'resolved')])),
            ],
            options={
                'ordering': ['id'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='HistoricalObservation',
            fields=[
                ('id', models.IntegerField(verbose_name='ID', db_index=True, auto_created=True, blank=True)),
                ('location_id', models.IntegerField(db_index=True, null=True, blank=True)),
                ('project_id', models.IntegerField(db_index=True, null=True, blank=True)),
                ('category_id', models.IntegerField(db_index=True, null=True, blank=True)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'draft', b'draft'), (b'review', b'review'), (b'pending', b'pending'), (b'deleted', b'deleted')])),
                ('attributes', django_hstore.fields.DictionaryField(db_index=True)),
                ('created_at', models.DateTimeField(editable=False, blank=True)),
                ('creator_id', models.IntegerField(db_index=True, null=True, blank=True)),
                ('updated_at', models.DateTimeField(null=True, editable=False, blank=True)),
                ('updator_id', models.IntegerField(db_index=True, null=True, blank=True)),
                ('version', models.IntegerField(default=1)),
                ('search_matches', models.TextField()),
                ('history_id', models.AutoField(serialize=False, primary_key=True)),
                ('history_date', models.DateTimeField()),
                ('history_type', models.CharField(max_length=1, choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')])),
            ],
            options={
                'ordering': ('-history_date', '-history_id'),
                'verbose_name': 'historical observation',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, null=True, blank=True)),
                ('description', models.TextField(null=True, blank=True)),
                ('geometry', django.contrib.gis.db.models.fields.GeometryField(srid=4326, geography=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('version', models.IntegerField(default=1)),
                ('private', models.BooleanField(default=False)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'review', b'review')])),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MediaFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(null=True, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'deleted', b'deleted')])),
            ],
            options={
                'ordering': ['id'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ImageFile',
            fields=[
                ('mediafile_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.MediaFile')),
                ('image', models.ImageField(upload_to=b'user-uploads/images')),
            ],
            options={
                'ordering': ['id'],
            },
            bases=('contributions.mediafile',),
        ),
        migrations.CreateModel(
            name='Observation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'draft', b'draft'), (b'review', b'review'), (b'pending', b'pending'), (b'deleted', b'deleted')])),
                ('attributes', django_hstore.fields.DictionaryField(db_index=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('version', models.IntegerField(default=1)),
                ('search_matches', models.TextField()),
                ('category', models.ForeignKey(to='categories.Category')),
            ],
            options={
                'ordering': ['-updated_at', 'id'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='VideoFile',
            fields=[
                ('mediafile_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.MediaFile')),
                ('video', models.ImageField(upload_to=b'user-uploads/videos')),
                ('youtube_id', models.CharField(max_length=100)),
                ('thumbnail', models.ImageField(null=True, upload_to=b'user-uploads/videos')),
                ('youtube_link', models.URLField(max_length=255, null=True, blank=True)),
                ('swf_link', models.URLField(max_length=255, null=True, blank=True)),
            ],
            options={
                'ordering': ['id'],
            },
            bases=('contributions.mediafile',),
        ),
    ]



from pytz import utc
from datetime import datetime, timedelta

from django.test import TestCase
from django.core import mail

from allauth.account.models import EmailAddress, EmailConfirmation

from geokey.projects.tests.model_factories import ProjectF, AdminsFactory
from geokey.contributions.tests.model_factories import ObservationFactory

from ..models import User
from ..management.commands.daily_digest import Command
from ..management.commands.check_confirm import Command as CheckConfirm
from ..tests.model_factories import UserF


class CheckConfirmTest(TestCase):
    def test_check_confirm(self):
        confirmed_user = UserF.create()
        unconfirmed_user = UserF.create()

        confirmed_email = EmailAddress.objects.create(
            user=confirmed_user,
            email=confirmed_user.email,
            verified=True
        )

        unconfirmed_email = EmailAddress.objects.create(
            user=unconfirmed_user,
            email=unconfirmed_user.email,
            verified=False
        )

        EmailConfirmation.objects.create(
            email_address=confirmed_email,
            created=datetime.utcnow().replace(tzinfo=utc),
            sent=datetime.utcnow().replace(tzinfo=utc),
            key='38974hewjkhnuweyf8h8d'
        )
        EmailConfirmation.objects.create(
            email_address=unconfirmed_email,
            created=datetime.utcnow().replace(tzinfo=utc),
            sent=datetime.utcnow().replace(tzinfo=utc),
            key='38974hewjkhnsdfuweyf8h8d'
        )

        yesterday = datetime.utcnow().replace(tzinfo=utc)
        command = CheckConfirm()
        command.set_user_inactive(yesterday)

        ref_unconfirmed = User.objects.get(pk=unconfirmed_user.id)
        self.assertFalse(ref_unconfirmed.is_active)

        ref_confirmed = User.objects.get(pk=confirmed_user.id)
        self.assertTrue(ref_confirmed.is_active)


class DailyDigestTest(TestCase):
    def test_get_update_projects(self):
        # Setup everything
        updated_project_1 = ProjectF.create()
        updated_project_2 = ProjectF.create()
        not_updated_project = ProjectF.create()

        for x in range(0, 2):
            for project in [
                    updated_project_1, updated_project_2, not_updated_project]:
                ObservationFactory.create(project=project)

        updated = ObservationFactory.create(project=updated_project_2)

        yesterday = datetime.utcnow().replace(tzinfo=utc)

        ObservationFactory.create(project=updated_project_1)
        updated.update(properties={'key': 'value'}, updator=UserF.create())

        # the tests
        command = Command()
        updated_projects = command.get_updated_projects(yesterday)
        self.assertIn(updated_project_1, updated_projects)
        self.assertIn(updated_project_2, updated_projects)
        self.assertNotIn(not_updated_project, updated_projects)

    def test_get_update_stats(self):
        moderator = UserF.create()
        contributor = UserF.create()
        some_dude = UserF.create()

        project = ProjectF.create(
            add_admins=[moderator],
            add_contributors=[contributor]
        )

        suspended = ObservationFactory.create(
            created_at=(datetime.utcnow() - timedelta(2)).replace(tzinfo=utc),
            project=project,
            creator=contributor,
            status='active'
        )

        updated = ObservationFactory.create(
            project=project,
            creator=contributor
        )

        reported = ObservationFactory.create(
            project=project,
            creator=contributor
        )

        approved = ObservationFactory.create(
            project=project,
            creator=contributor,
            status='pending'
        )

        yesterday = datetime.utcnow().replace(tzinfo=utc)

        suspended.update(
            properties=None,
            status='pending',
            updator=UserF.create()
        )

        reported.update(
            properties=None,
            status='review',
            updator=UserF.create()
        )

        updated.update(properties={'key': 'value'}, updator=UserF.create())
        approved.update(properties=None, status='active', updator=moderator)

        for user in [moderator, contributor]:
            ObservationFactory.create(
                project=project,
                creator=user,
                status='pending'
            )

        command = Command()

        report = command.get_updated_items(project, moderator, yesterday)
        to_moderate = report.get('to_moderate')
        self.assertEqual(len(to_moderate.get('new')), 2)
        self.assertEqual(len(to_moderate.get('suspended')), 1)
        self.assertIsNone(report.get('yours'))

        report = command.get_updated_items(project, contributor, yesterday)

        yours = report.get('yours')
        self.assertEqual(len(yours.get('changed')), 4)
        self.assertEqual(len(yours.get('approved')), 1)
        self.assertEqual(len(yours.get('reported')), 1)
        self.assertEqual(len(yours.get('suspended')), 1)

        report = command.get_updated_items(project, some_dude, yesterday)
        self.assertEqual(report, None)

    def test_daily_digest(self):
        do_not_notify = UserF.create()
        moderator = UserF.create()
        contributor = UserF.create()
        UserF.create()

        project = ProjectF.create(
            add_admins=[moderator],
            add_contributors=[contributor]
        )
        AdminsFactory.create(**{
            'project': project,
            'user': do_not_notify,
            'contact': False
        })

        suspended = ObservationFactory.create(
            created_at=(datetime.utcnow() - timedelta(2)).replace(tzinfo=utc),
            project=project,
            creator=contributor,
            status='active'
        )

        updated = ObservationFactory.create(
            project=project,
            creator=contributor
        )

        approved = ObservationFactory.create(
            project=project,
            creator=contributor,
            status='pending'
        )

        yesterday = datetime.utcnow().replace(tzinfo=utc)

        suspended.update(
            properties=None,
            status='pending',
            updator=UserF.create()
        )
        updated.update(properties={'key': 'value'}, updator=UserF.create())
        approved.update(properties=None, status='active', updator=moderator)

        for user in [moderator, contributor]:
            ObservationFactory.create(
                project=project,
                creator=user,
                status='pending'
            )

        command = Command()
        command.send_daily_digest(yesterday)
        self.assertEquals(len(mail.outbox), 3)



from django.test import TestCase
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied

from geokey.users.tests.model_factories import UserF, UserGroupF

from .model_factories import ProjectF
from ..models import Project


class ProjectListTest(TestCase):
    def setUp(self):
        self.admin = UserF.create()
        self.moderator_view = UserF.create()
        self.moderator = UserF.create()
        self.contributor_view = UserF.create()
        self.contributor = UserF.create()
        self.viewer = UserF.create()
        self.some_dude = UserF.create()

        self.private_project = ProjectF.create(
            add_admins=[self.admin],
            **{
                'name': 'Private',
                'isprivate': True,
                'everyone_contributes': 'false'
            })

        self.private_moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.private_project,
                'can_moderate': True
            })
        self.private_contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.private_project,
                'can_contribute': True
            })

        self.private_moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.private_project,
                'can_moderate': True
            })
        self.private_contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.private_project,
                'can_contribute': True
            })
        self.private_viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.private_project,
                'can_contribute': False,
                'can_moderate': False
            })

        self.private_publicview_project = ProjectF.create(
            add_admins=[self.admin],
            **{
                'name': 'Private with public view',
                'isprivate': True,
                'everyone_contributes': 'false'
            })

        self.private_publicview_moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.private_publicview_project,
                'can_moderate': True
            })
        self.private_publicview_contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.private_publicview_project,
                'can_contribute': True
            })

        self.private_publicview_moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.private_publicview_project,
                'can_moderate': True
            })
        self.private_publicview_contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.private_publicview_project,
                'can_contribute': True
            })
        self.private_publicview_viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.private_publicview_project,
                'can_contribute': False,
                'can_moderate': False
            })

        self.private_publicviews_project = ProjectF.create(
            add_admins=[self.admin],
            **{
                'name': 'Private with public views',
                'isprivate': True,
                'everyone_contributes': 'false'
            })

        self.private_publicviews_moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.private_publicviews_project,
                'can_moderate': True
            })
        self.private_publicviews_contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.private_publicviews_project,
                'can_contribute': True
            })

        self.private_publicviews_moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.private_publicviews_project,
                'can_moderate': True
            })
        self.private_publicviews_contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.private_publicviews_project,
                'can_contribute': True
            })
        self.private_publicviews_viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.private_publicviews_project,
                'can_contribute': False,
                'can_moderate': False
            })

        self.public_project = ProjectF.create(
            add_admins=[self.admin],
            **{
                'name': 'Public',
                'isprivate': False,
                'everyone_contributes': 'false'
            })

        self.public_moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.public_project,
                'can_moderate': True
            })
        self.public_contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.public_project,
                'can_contribute': True
            })

        self.public_moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.public_project,
                'can_moderate': True
            })
        self.public_contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.public_project,
                'can_contribute': True
            })
        self.public_viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.public_project,
                'can_contribute': False,
                'can_moderate': False
            })

        self.public_publicview_project = ProjectF.create(
            add_admins=[self.admin],
            **{
                'name': 'Public with public view',
                'isprivate': False,
                'everyone_contributes': 'false'
            })

        self.public_publicview_moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.public_publicview_project,
                'can_moderate': True
            })
        self.public_publicview_contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.public_publicview_project,
                'can_contribute': True
            })

        self.public_publicview_moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.public_publicview_project,
                'can_moderate': True
            })
        self.public_publicview_contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.public_publicview_project,
                'can_contribute': True
            })
        self.public_publicview_viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.public_publicview_project,
                'can_contribute': False,
                'can_moderate': False
            })

        self.public_publicviews_project = ProjectF.create(
            add_admins=[self.admin],
            **{
                'name': 'public with all public views',
                'isprivate': False,
                'everyone_contributes': 'false'
            })

        self.public_publicviews_moderators_view = UserGroupF(
            add_users=[self.moderator_view],
            **{
                'project': self.public_publicviews_project,
                'can_moderate': True
            })
        self.public_publicviews_contributors_view = UserGroupF(
            add_users=[self.contributor_view],
            **{
                'project': self.public_publicviews_project,
                'can_contribute': True
            })

        self.public_publicviews_moderators = UserGroupF(
            add_users=[self.moderator],
            **{
                'project': self.public_publicviews_project,
                'can_moderate': True
            })
        self.public_publicviews_contributors = UserGroupF(
            add_users=[self.contributor],
            **{
                'project': self.public_publicviews_project,
                'can_contribute': True
            })
        self.public_publicviews_viewers = UserGroupF(
            add_users=[self.viewer],
            **{
                'project': self.public_publicviews_project,
                'can_contribute': False,
                'can_moderate': False
            })

    def test_get_list_with_admin(self):
        projects = Project.objects.get_list(self.admin)
        self.assertEquals(len(projects), 6)

    def test_get_list_with_moderator_view(self):
        projects = Project.objects.get_list(self.moderator_view)
        self.assertEquals(len(projects), 6)

    def test_get_list_with_moderator(self):
        projects = Project.objects.get_list(self.moderator)
        self.assertEquals(len(projects), 6)

    def test_get_list_with_contributor_view(self):
        projects = Project.objects.get_list(self.contributor_view)
        self.assertEquals(len(projects), 6)

    def test_get_list_with_contributor(self):
        projects = Project.objects.get_list(self.contributor)
        self.assertEquals(len(projects), 6)

    def test_get_list_with_viewer(self):
        projects = Project.objects.get_list(self.viewer)
        self.assertEquals(len(projects), 6)

    def test_get_list_with_some_dude(self):
        projects = Project.objects.get_list(self.some_dude)
        self.assertEquals(len(projects), 3)

    def test_get_list_with_anonymous(self):
        projects = Project.objects.get_list(AnonymousUser())
        self.assertEquals(len(projects), 3)


class ProjectAccessTest(TestCase):
    def test_as_admin(self):
        project = ProjectF.create()
        user = UserF.create(**{'is_superuser': True})
        ref = Project.objects.as_admin(user, project.id)
        self.assertEqual(project, ref)

    def test_as_contributor(self):
        admin = UserF.create()
        contributor = UserF.create()
        other = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_contributors=[contributor],
            add_viewer=[other]
        )

        ref = Project.objects.as_contributor(admin, project.id)
        self.assertEqual(project, ref)

        ref = Project.objects.as_contributor(contributor, project.id)
        self.assertEqual(project, ref)

        try:
            Project.objects.as_contributor(other, project.id)
        except PermissionDenied:
            pass
        else:
            self.fail('PermissionDenied not raise for non contributor')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django_pgjson.fields


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_auto_20150106_1420'),
    ]

    operations = [
        migrations.AddField(
            model_name='usergroup',
            name='filters',
            field=django_pgjson.fields.JsonBField(null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='usergroup',
            name='where_clause',
            field=models.TextField(null=True, blank=True),
            preserve_default=True,
        ),
    ]



#!/usr/bin/python

import sys
import os

from django.conf import settings
from django.contrib.sites.models import Site

from datetime import datetime, timedelta
from pytz import utc

from django.core.management.base import NoArgsCommand
from django.db.models import Q
from django.core import mail

from django.template.loader import get_template
from django.template import Context

from geokey.projects.models import Project, Admins
from geokey.users.models import User
from geokey.contributions.models import Observation


class Command(NoArgsCommand):
    def get_updated_projects(self, yesterday):
        """
        Returns all projects that where updated during the previous day
        """
        projects = Project.objects.filter(status='active')
        return projects.filter(
            Q(observations__created_at__gte=yesterday) |
            Q(observations__updated_at__gte=yesterday)).distinct()

    def get_updated_items(self, project, user, yesterday):
        """
        Returns all updated contributions of a project for the given users.

        Returns a dict if the user is moderator or contributor, `to_moderate`
        is only available when the user is a moderator:
        {
            'to_moderate': {
                'new': [],
                'reported': []
            },
            'yours': {
                'changed': []
                'approved': []
                'reports': []
            }
        }

        Returns `None` if the user is neither moderator or contributor of the
        project.
        """
        items = {}

        try:
            notify = Admins.objects.get(user=user, project=project).contact
        except Admins.DoesNotExist:
            notify = True

        if notify:
            new_items = project.observations.filter(
                created_at__gte=yesterday, status='pending')
            updated_items = project.observations.filter(
                updated_at__gte=yesterday,
                created_at__lt=yesterday
            )

            if updated_items.count() > 0 or new_items.count() > 0:

                if project.can_moderate(user):
                    items['to_moderate'] = {
                        'new': new_items,
                        'reported': updated_items.filter(status='review'),
                        'suspended': updated_items.filter(status='pending')
                    }
                if project.can_contribute(user):
                    approved = []
                    reported = []
                    suspended = []

                    for item in updated_items.filter(creator=user):
                        try:
                            historical = item.history.as_of(yesterday)

                            if (item.status == 'pending' and
                                    historical.status == 'active'):
                                suspended.append(item)
                            elif (item.status == 'review' and
                                    historical.status == 'active'):
                                reported.append(item)
                            elif (item.status == 'active' and
                                    historical.status == 'pending'):
                                approved.append(item)
                        except Observation.DoesNotExist:
                            pass

                        items['yours'] = {
                            'changed': updated_items.filter(creator=user),
                            'approved': approved,
                            'reported': reported,
                            'suspended': suspended
                        }
                    return items

        return None

    def send_daily_digest(self, yesterday):
        """
        Creates a daily digest for all users registered in the system and sends
        the digest to theses users. Digests will only be sent if there are
        relevant updates in at least one project.
        """
        messages = []

        updated_projects = self.get_updated_projects(yesterday)
        platform = Site.objects.get(pk=settings.SITE_ID).name

        for user in User.objects.exclude(display_name='AnonymousUser'):
            reports = []
            for project in updated_projects:
                updated_items = self.get_updated_items(
                    project,
                    user,
                    yesterday
                )

                if updated_items is not None:
                    to_moderate = updated_items.pop('to_moderate', None)
                    yours = updated_items.pop('yours', None)

                    if to_moderate is not None or yours is not None:
                        reports.append({
                            'project': project.name,
                            'to_moderate': to_moderate,
                            'yours': yours
                        })

            if len(reports) > 0:
                email_text = get_template('users/daily_digest.txt')
                context = Context({
                    'user': user.display_name,
                    'yesterday': yesterday.strftime('%d %B %Y'),
                    'reports': reports,
                    'platform': platform
                })
                text = email_text.render(context)

                email = mail.EmailMessage(
                    '%s daily digest for %s' % (
                        platform,
                        yesterday.strftime('%d %B %Y')
                    ),
                    text,
                    settings.DEFAULT_FROM_EMAIL,
                    [user.email]
                )
                messages.append(email)

        if len(messages) > 0:
            connection = mail.get_connection()
            connection.open()
            connection.send_messages(messages)
            connection.close()

    def handle(self, *args, **options):
        now = datetime.utcnow() - timedelta(1)
        yesterday = datetime(
            now.year, now.month, now.day, 0, 0, 0).replace(tzinfo=utc)

        self.send_daily_digest(yesterday)



from django.db import models

from oauth2_provider.models import AbstractApplication

from .base import STATUS
from .manager import ApplicationManager


class Application(AbstractApplication):
    """
    Represents an OAuth client an application developer resgisters with GeoKey
    in order to obtain OAuth tokens to interact with GeoKey.
    """
    description = models.TextField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    download_url = models.URLField(blank=False)
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )
    objects = ApplicationManager()



from django.test import TestCase

from geokey.contributions.models import Observation

from geokey.projects.tests.model_factories import ProjectF, UserF
from geokey.categories.tests.model_factories import (
    CategoryFactory, LookupFieldFactory, LookupValueFactory,
    TextFieldFactory, MultipleLookupFieldFactory, MultipleLookupValueFactory
)
from ..model_factories import ObservationFactory


class ObservationTest(TestCase):
    def setUp(self):
        self.creator = UserF.create()
        ObservationFactory.create_batch(
            5, **{'status': 'active', 'creator': self.creator})
        ObservationFactory.create_batch(
            5, **{'status': 'draft', 'creator': self.creator})
        ObservationFactory.create_batch(
            5, **{'status': 'pending', 'creator': self.creator})
        ObservationFactory.create_batch(
            5, **{'status': 'deleted', 'creator': self.creator})

    def test_for_creator_moderator(self):
        observations = Observation.objects.all().for_moderator(self.creator)
        self.assertEqual(len(observations), 15)
        for observation in observations:
            self.assertNotIn(
                observation.status,
                ['deleted']
            )

    def test_for_moderator(self):
        observations = Observation.objects.all().for_moderator(UserF.create())
        self.assertEqual(len(observations), 10)
        for observation in observations:
            self.assertNotIn(
                observation.status,
                ['draft', 'deleted']
            )

    def test_for_viewer(self):
        observations = Observation.objects.all().for_viewer(UserF.create())
        self.assertEqual(len(observations), 5)
        for observation in observations:
            self.assertNotIn(
                observation.status,
                ['draft', 'pending', 'deleted']
            )


class TestSearch(TestCase):
    def setUp(self):
        o_type = CategoryFactory.create()
        TextFieldFactory.create(**{'key': 'key', 'category': o_type})

        ObservationFactory.create_batch(5, **{
            'properties': {'key': 'blah'},
            'category': o_type
        })
        ObservationFactory.create_batch(5, **{
            'properties': {'key': 'blub'},
            'category': o_type
        })

    def test_bl(self):
        result = Observation.objects.all().search('bl')
        self.assertEqual(len(result), 10)

    def test_blah(self):
        result = Observation.objects.all().search('blah')
        self.assertEqual(len(result), 5)

        for o in result:
            self.assertEqual(o.properties.get('key'), 'blah')

    def test_blub(self):
        result = Observation.objects.all().search('blub')
        self.assertEqual(len(result), 5)

        for o in result:
            self.assertEqual(o.properties.get('key'), 'blub')

    def test_single_lookup(self):
        project = ProjectF.create()
        category = CategoryFactory.create(**{'project': project})
        lookup = LookupFieldFactory.create(
            **{'category': category, 'key': 'lookup'}
        )
        kermit = LookupValueFactory.create(**{
            'field': lookup,
            'name': 'Kermit'
        })
        gonzo = LookupValueFactory.create(**{
            'field': lookup,
            'name': 'Gonzo'
        })
        ObservationFactory.create_batch(3, **{
            'project': project,
            'category': category,
            'properties': {'lookup': kermit.id}
        })
        ObservationFactory.create_batch(3, **{
            'project': project,
            'category': category,
            'properties': {'lookup': gonzo.id}
        })

        result = project.observations.all().search('kermit')
        self.assertEqual(len(result), 3)

        for o in result:
            self.assertEqual(o.properties.get('lookup'), kermit.id)

    def test_multiple_lookup(self):
        project = ProjectF.create()
        category = CategoryFactory.create(**{'project': project})
        lookup = MultipleLookupFieldFactory.create(
            **{'category': category, 'key': 'lookup'}
        )
        kermit = MultipleLookupValueFactory.create(**{
            'field': lookup,
            'name': 'Kermit'
        })
        gonzo = MultipleLookupValueFactory.create(**{
            'field': lookup,
            'name': 'Gonzo'
        })
        piggy = MultipleLookupValueFactory.create(**{
            'field': lookup,
            'name': 'Ms Piggy'
        })
        ObservationFactory.create_batch(3, **{
            'project': project,
            'category': category,
            'properties': {'lookup': [piggy.id, kermit.id]}
        })
        ObservationFactory.create_batch(3, **{
            'project': project,
            'category': category,
            'properties': {'lookup': [gonzo.id]}
        })

        result = project.observations.all().search('kermit')
        self.assertEqual(len(result), 3)

        for o in result:
            self.assertIn(kermit.id, o.properties.get('lookup'))

        result = project.observations.all().search('piggy')
        self.assertEqual(len(result), 3)

        for o in result:
            self.assertIn(kermit.id, o.properties.get('lookup'))



from django.test import TestCase

from nose.tools import raises

from geokey.contributions.models import Comment
from ..model_factories import CommentFactory


class CommentTest(TestCase):
    @raises(Comment.DoesNotExist)
    def test_delete_comment(self):
        comment = CommentFactory()
        comment.delete()
        Comment.objects.get(pk=comment.id)

    def test_delete_nested(self):
        comment = CommentFactory()
        response = CommentFactory(**{'respondsto': comment})
        CommentFactory.create_batch(3, **{'respondsto': response})
        self.assertEquals(len(Comment.objects.all()), 5)
        comment.delete()
        self.assertEquals(len(Comment.objects.all()), 0)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('projects', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='admins',
            field=models.ManyToManyField(related_name='admins', through='projects.Admins', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='project',
            name='creator',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='admins',
            name='project',
            field=models.ForeignKey(related_name='admin_of', to='projects.Project'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='admins',
            name='user',
            field=models.ForeignKey(related_name='has_admins', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='admins',
            unique_together=set([('project', 'user')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.contrib.gis.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Admins',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('contact', models.BooleanField(default=True)),
            ],
            options={
                'ordering': ['project__name'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(null=True, blank=True)),
                ('isprivate', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('everyone_contributes', models.BooleanField(default=True)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'inactive', b'inactive'), (b'deleted', b'deleted')])),
                ('geographic_extend', django.contrib.gis.db.models.fields.PolygonField(srid=4326, null=True, geography=True)),
            ],
            options={
                'ordering': ['name'],
            },
            bases=(models.Model,),
        ),
    ]



import json

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AnonymousUser
from django.http import HttpResponseRedirect
from django.contrib.gis.geos import GEOSGeometry
from django.core import mail

from rest_framework.test import APIRequestFactory, force_authenticate

from geokey.categories.tests.model_factories import (
    TextFieldFactory, CategoryFactory
)
from geokey.users.tests.model_factories import UserF, UserGroupF

from .model_factories import ProjectF
from ..models import Project
from ..views import (
    ProjectCreate, ProjectSettings, ProjectUpdate, ProjectAdmins,
    ProjectAdminsUser, Projects, SingleProject, ProjectOverview, ProjectExtend,
    ProjectContactAdmins, CategoriesReorderView, ProjectsInvolved,
    ProjectDelete
)

# ############################################################################
#
# ADMIN VIEWS
#
# ############################################################################


class ProjectCreateTest(TestCase):
    def test_get_with_user(self):
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().get(url)
        request.user = UserF.create()
        response = view(request).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_post_with_user(self):
        data = {
            'name': 'Project',
            'description': '',
            'isprivate': True,
            'everyone_contributes': 'auth'
        }
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().post(url, data)
        request.user = UserF.create()

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 1)

    def test_post_with_anonymous(self):
        data = {
            'name': 'Project',
            'description': '',
            'isprivate': True,
            'everyone_contributes': 'auth'
        }
        view = ProjectCreate.as_view()
        url = reverse('admin:project_create')
        request = APIRequestFactory().post(url, data)
        request.user = AnonymousUser()

        response = view(request)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 0)


class ProjectsInvolvedTest(TestCase):
    def test_get_with_user(self):
        user = UserF.create()
        ProjectF.create(add_contributors=[user])
        view = ProjectsInvolved.as_view()
        url = reverse('admin:projects_involved')
        request = APIRequestFactory().get(url)
        request.user = user
        response = view(request).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectsInvolved.as_view()
        url = reverse('admin:projects_involved')
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertEqual(response.status_code, 302)


class ProjectExtendTest(TestCase):
    def setUp(self):
        self.creator = UserF.create()
        self.admin = UserF.create()
        self.view_member = UserF.create()
        self.contributor = UserF.create()
        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{
                'creator': self.creator
            }
        )

    def test_get_with_creator(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.creator
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_admin(self):
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.contributor
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_view_member(self):
        view = ProjectExtend.as_view()
        url = reverse(
            'admin:project_extend',
            kwargs={'project_id': self.project.id}
        )
        request = APIRequestFactory().get(url)
        request.user = self.view_member
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectExtend.as_view()
        url = reverse(
            'admin:project_extend',
            kwargs={'project_id': self.project.id}
        )
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_get_deleted_project(self):
        self.project.delete()
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Project matching query does not exist.')

    def test_update(self):
        data = {'geometry': '{"type": "Polygon","coordinates": [['
                            '[-0.508,51.682],[-0.53,51.327],[0.225,51.323],'
                            '[0.167,51.667],[-0.508,51.682]]]}'}
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.admin

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, 'The geographic extent has been updated successfully.')

        updated = Project.objects.get(pk=self.project.id)

        self.assertEqual(updated.geographic_extend.geom_type, 'Polygon')
        self.assertEqual(
            updated.geographic_extend.json,
            GEOSGeometry(data.get('geometry')).json
        )

    def test_update_with_none(self):
        data = None
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.admin

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, 'The geographic extent has been updated successfully.')

        updated = Project.objects.get(pk=self.project.id)

        self.assertEqual(updated.geographic_extend, None)

    def test_update_with_empty_string(self):
        data = {'geometry': ''}
        view = ProjectExtend.as_view()
        url = reverse('admin:project_extend',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.admin

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, 'The geographic extent has been updated successfully.')

        updated = Project.objects.get(pk=self.project.id)

        self.assertEqual(updated.geographic_extend, None)


class ProjectSettingsTest(TestCase):
    def setUp(self):
        self.creator = UserF.create()
        self.admin = UserF.create()
        self.view_member = UserF.create()
        self.contributor = UserF.create()
        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{'creator': self.creator}
        )

    def test_get_with_creator(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.creator
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_admin(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.contributor
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_view_member(self):
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.view_member
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectSettings.as_view()
        url = reverse(
            'admin:project_extend',
            kwargs={'project_id': self.project.id}
        )
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_get_deleted_project(self):
        self.project.delete()
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Project matching query does not exist.')

    def test_post_with_admin(self):
        data = {
            'name': 'Project',
            'description': '',
            'isprivate': True,
            'everyone_contributes': 'auth'
        }
        view = ProjectSettings.as_view()
        url = reverse('admin:project_settings',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().post(url, data)
        request.user = self.creator

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)


class ProjectOverviewTest(TestCase):
    def setUp(self):
        self.creator = UserF.create()
        self.admin = UserF.create()
        self.view_member = UserF.create()
        self.contributor = UserF.create()
        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{
                'creator': self.creator
            }
        )

    def test_get_with_admin(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_contributor(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.contributor
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_view_member(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.view_member
        response = view(request, project_id=self.project.id)
        self.assertEqual(response.status_code, 200)

    def test_get_with_anonymous(self):
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_get_deleted_project(self):
        self.project.delete()
        view = ProjectOverview.as_view()
        url = reverse('admin:project_overview',
                      kwargs={'project_id': self.project.id})
        request = APIRequestFactory().get(url)
        request.user = self.admin
        response = view(request, project_id=self.project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Project matching query does not exist.')


class ProjectDeleteTest(TestCase):
    def test_delete_with_admin(self):
        user = UserF.create()
        project = ProjectF.create(add_admins=[user])

        view = ProjectDelete.as_view()
        url = reverse('admin:project_delete',
                      kwargs={'project_id': project.id})
        request = APIRequestFactory().get(url)
        request.user = user

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request, project_id=project.id)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 0)

    def test_delete_with_contributor(self):
        user = UserF.create()
        project = ProjectF.create(add_contributors=[user])

        view = ProjectDelete.as_view()
        url = reverse('admin:project_delete',
                      kwargs={'project_id': project.id})
        request = APIRequestFactory().get(url)
        request.user = user
        response = view(request, project_id=project.id)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(Project.objects.count(), 1)

    def test_delete_with_anonymous(self):
        project = ProjectF.create()

        view = ProjectDelete.as_view()
        url = reverse('admin:project_delete',
                      kwargs={'project_id': project.id})
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request, project_id=project.id)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Project.objects.count(), 1)


# ############################################################################
#
# AJAX VIEWS
#
# ############################################################################

class ProjectUpdateTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()

        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

    def test_unauthenticated(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'status': 'bockwurst'}
        )
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_update_with_wrong_status(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'status': 'bockwurst'}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 400)

    def test_update_project_status_with_admin(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'status': 'inactive'}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).status,
            'inactive'
        )

    def test_update_project_description_with_admin(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'description': 'new description'}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).description,
            'new description'
        )

    def test_update_project_description_with_contributor(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'description': 'new description'}
        )
        force_authenticate(request, user=self.contributor)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 403)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).description,
            self.project.description
        )

    def test_update_project_description_with_non_member(self):
        request = self.factory.put(
            '/api/projects/%s/' % self.project.id,
            {'description': 'new description'}
        )
        force_authenticate(request, user=self.non_member)
        view = ProjectUpdate.as_view()
        response = view(request, project_id=self.project.id).render()

        self.assertEqual(response.status_code, 404)
        self.assertEqual(
            Project.objects.get(pk=self.project.id).description,
            self.project.description
        )


class ProjectAdminsTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()
        self.user_to_add = UserF.create()

        self.project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

    def test_add_not_existing_user(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': 468476351545643131}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 400)

    def test_add_admin_with_admin(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': self.user_to_add.id}
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 201)
        self.assertIn(
            self.user_to_add,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_add_admin_with_contributor(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': self.user_to_add.id}
        )
        force_authenticate(request, user=self.contributor)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 403)
        self.assertNotIn(
            self.user_to_add,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_add_admin_with_non_member(self):
        request = self.factory.post(
            '/ajax/projects/%s/admins/' % (self.project.id) + '/',
            {'userId': self.user_to_add.id}
        )
        force_authenticate(request, user=self.non_member)
        view = ProjectAdmins.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 404)
        self.assertNotIn(
            self.user_to_add,
            Project.objects.get(pk=self.project.id).admins.all()
        )


class ProjectAdminsUserTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()
        self.admin_to_remove = UserF.create()

        self.project = ProjectF.create(
            add_admins=[self.admin, self.admin_to_remove],
            add_contributors=[self.contributor]
        )

    def test_delete_not_existing_admin(self):
        user = UserF.create()
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, user.id)
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=user.id
        ).render()
        self.assertEqual(response.status_code, 404)

    def test_delete_adminuser_with_admin(self):
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, self.admin_to_remove.id)
        )
        force_authenticate(request, user=self.admin)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=self.admin_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 204)
        self.assertNotIn(
            self.admin_to_remove,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_delete_adminuser_with_contributor(self):
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, self.admin_to_remove.id)
        )
        force_authenticate(request, user=self.contributor)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=self.admin_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 403)
        self.assertIn(
            self.admin_to_remove,
            Project.objects.get(pk=self.project.id).admins.all()
        )

    def test_delete_adminuser_with_non_member(self):
        request = self.factory.delete(
            '/ajax/projects/%s/admins/%s/' %
            (self.project.id, self.admin_to_remove.id)
        )
        force_authenticate(request, user=self.non_member)
        view = ProjectAdminsUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            user_id=self.admin_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 404)
        self.assertIn(
            self.admin_to_remove,
            Project.objects.get(pk=self.project.id).admins.all()
        )


class ReorderCategoriesTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.project = ProjectF.create()

        self.category_0 = CategoryFactory.create(**{'project': self.project})
        self.category_1 = CategoryFactory.create(**{'project': self.project})
        self.category_2 = CategoryFactory.create(**{'project': self.project})
        self.category_3 = CategoryFactory.create(**{'project': self.project})
        self.category_4 = CategoryFactory.create(**{'project': self.project})

    def test_reorder(self):
        url = reverse(
            'ajax:categories_reorder',
            kwargs={
                'project_id': self.project.id
            }
        )

        data = [
            self.category_4.id, self.category_0.id, self.category_2.id,
            self.category_1.id, self.category_3.id
        ]

        request = self.factory.post(
            url, json.dumps({'order': data}), content_type='application/json')
        force_authenticate(request, user=self.project.creator)
        view = CategoriesReorderView.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 200)

        categories = self.project.categories.all()

        self.assertTrue(categories.ordered)
        self.assertEqual(categories[0], self.category_4)
        self.assertEqual(categories[1], self.category_0)
        self.assertEqual(categories[2], self.category_2)
        self.assertEqual(categories[3], self.category_1)
        self.assertEqual(categories[4], self.category_3)

    def test_reorder_with_false_category(self):
        url = reverse(
            'ajax:categories_reorder',
            kwargs={
                'project_id': self.project.id
            }
        )

        data = [
            self.category_4.id, self.category_0.id, self.category_2.id,
            self.category_1.id, 655123135135
        ]

        request = self.factory.post(
            url, json.dumps({'order': data}), content_type='application/json')
        force_authenticate(request, user=self.project.creator)
        view = CategoriesReorderView.as_view()
        response = view(
            request,
            project_id=self.project.id
        ).render()

        self.assertEqual(response.status_code, 400)

        categories = self.project.categories.all()
        self.assertTrue(categories.ordered)
        self.assertEqual(categories[0].order, 0)
        self.assertEqual(categories[1].order, 1)
        self.assertEqual(categories[2].order, 2)
        self.assertEqual(categories[3].order, 3)
        self.assertEqual(categories[4].order, 4)


# ############################################################################
#
# API VIEWS
#
# ############################################################################

class ProjectsTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.contributor = UserF.create()
        self.non_member = UserF.create()

        self.public_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{
                'isprivate': False
            }
        )

        self.private_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )

        self.inactive_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{
                'status': 'inactive'
            }
        )

        self.deleted_project = ProjectF.create(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{'isprivate': False}
        )
        self.deleted_project.delete()

    def test_get_projects_with_admin(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=self.admin)
        view = Projects.as_view()
        response = view(request).render()
        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 2)
        self.assertNotContains(response, self.deleted_project.name)

    def test_get_projects_with_contributor(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=self.contributor)
        view = Projects.as_view()
        response = view(request).render()

        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 2)
        self.assertNotContains(response, self.inactive_project.name)
        self.assertNotContains(response, self.deleted_project.name)

    def test_get_projects_with_non_member(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=self.non_member)
        view = Projects.as_view()
        response = view(request).render()

        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 1)
        self.assertNotContains(response, self.inactive_project.name)
        self.assertNotContains(response, self.deleted_project.name)
        self.assertNotContains(response, self.private_project.name)

    def test_get_projects_with_anonymous(self):
        request = self.factory.get('/api/projects/')
        force_authenticate(request, user=AnonymousUser())
        view = Projects.as_view()
        response = view(request).render()

        projects = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(projects), 1)
        self.assertNotContains(response, self.inactive_project.name)
        self.assertNotContains(response, self.deleted_project.name)
        self.assertNotContains(response, self.private_project.name)


class SingleProjectTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()

    def test_category_serialization(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user]
        )
        CategoryFactory.create(**{'project': project})
        CategoryFactory.create(
            **{'project': project, 'status': 'inactive'}
        )
        o1 = CategoryFactory.create(**{'project': project})
        TextFieldFactory.create(**{'category': o1})
        o2 = CategoryFactory.create(**{'project': project})
        TextFieldFactory.create(**{'category': o2})

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            2,
            len(json.loads(response.content).get('categories'))
        )

    def test_get_deleted_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user]
        )
        project.delete()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_private_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user]
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')
        self.assertContains(response, '"is_involved":true')
        self.assertContains(response, '"geographic_extent"')

    def test_get_inactive_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user],
            **{'status': 'inactive'}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 403)

    def test_get_public_project_with_admin(self):
        user = UserF.create()

        project = ProjectF.create(
            add_admins=[user],
            **{'isprivate': False}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')

    def test_get_deleted_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user]
        )
        project.delete()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_private_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user]
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')

    def test_get_inactive_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user],
            **{'status': 'inactive'}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_public_project_with_contributor(self):
        user = UserF.create()

        project = ProjectF.create(
            add_contributors=[user],
            **{'isprivate': False}
        )

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":true')

    def test_get_deleted_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create()
        project.delete()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_private_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create()

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_inactive_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create(**{
            'status': 'inactive'
        })

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 404)

    def test_get_public_project_with_non_member(self):
        user = UserF.create()

        project = ProjectF.create(**{
            'isprivate': False
        })

        request = self.factory.get(
            '/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":false')

    def test_get_public_project_with_anonymous(self):
        user = AnonymousUser()

        project = ProjectF.create(**{
            'isprivate': False
        })

        request = self.factory.get('/api/projects/%s/' % project.id)
        force_authenticate(request, user=user)
        view = SingleProject.as_view()
        response = view(request, project_id=project.id).render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, project.name)
        self.assertContains(response, '"can_contribute":false')


class ProjectContactAdminsTest(TestCase):
    def test_with_active_project(self):
        admin = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            add_viewer=[email_user]
        )

        view = ProjectContactAdmins.as_view()
        url = reverse(
            'api:project_contact_admins',
            kwargs={
                'project_id': project.id
            }
        )
        request = APIRequestFactory().post(
            url,
            json.dumps({'email_text': 'Blah Blah'}),
            content_type='application/json'
        )
        force_authenticate(request, user=email_user)

        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 204)
        self.assertEquals(len(mail.outbox), 2)

    def test_with_inactive_project(self):
        admin = UserF.create()
        email_user = UserF.create()

        project = ProjectF.create(
            add_admins=[admin],
            **{'status': 'inactive'}
        )

        view = ProjectContactAdmins.as_view()
        url = reverse(
            'api:project_contact_admins',
            kwargs={
                'project_id': project.id
            }
        )
        request = APIRequestFactory().post(
            url,
            json.dumps({'email_text': 'Blah Blah'}),
            content_type='application/json'
        )
        force_authenticate(request, user=email_user)

        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 404)
        self.assertEquals(len(mail.outbox), 0)

    def test_with_anonymous(self):
        admin = UserF.create()
        email_user = AnonymousUser()

        project = ProjectF.create(
            add_admins=[admin],
            **{'isprivate': False}
        )

        view = ProjectContactAdmins.as_view()
        url = reverse(
            'api:project_contact_admins',
            kwargs={
                'project_id': project.id
            }
        )
        request = APIRequestFactory().post(
            url,
            json.dumps({'email_text': 'Blah Blah'}),
            content_type='application/json'
        )
        force_authenticate(request, user=email_user)

        response = view(request, project_id=project.id).render()
        self.assertEqual(response.status_code, 401)
        self.assertEquals(len(mail.outbox), 0)



import os
import glob

from django.test import TestCase
from django.conf import settings
from django.contrib.auth.models import AnonymousUser

from geokey.users.tests.model_factories import UserF

from .model_factories import ImageFileFactory, VideoFileFactory
from geokey.contributions.serializers import FileSerializer


class FileSerializerTest(TestCase):
    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def test_get_is_owner(self):
        image = ImageFileFactory.create()

        serializer = FileSerializer(image, context={'user': image.creator})
        self.assertTrue(serializer.get_isowner(image))

        serializer = FileSerializer(image, context={'user': UserF.create()})
        self.assertFalse(serializer.get_isowner(image))

        serializer = FileSerializer(image, context={'user': AnonymousUser()})
        self.assertFalse(serializer.get_isowner(image))

    def test_get_image_url(self):
        image = ImageFileFactory.create()

        serializer = FileSerializer(image, context={'user': image.creator})
        self.assertEqual(serializer.get_url(image), image.image.url)

    def test_get_thumb_url(self):
        image = ImageFileFactory.create()

        serializer = FileSerializer(image, context={'user': image.creator})
        self.assertEqual(
            serializer.get_thumbnail_url(image),
            image.image.url + '.300x300_q85_crop.png'
        )

    def test_get_youtube_link(self):
        video = VideoFileFactory.create()

        serializer = FileSerializer(video, context={'user': video.creator})
        self.assertEqual(serializer.get_url(video), video.youtube_link)

    def test_get_youtube_no_thumb(self):
        video = VideoFileFactory.create(**{'youtube_id': 'asadf'})

        serializer = FileSerializer(video, context={'user': video.creator})
        self.assertEqual(
            serializer.get_thumbnail_url(video),
            '/static/img/play.png'
        )

    def test_get_youtube_thumb(self):
        video = VideoFileFactory.create(**{'youtube_id': '14emk_jPnrI'})

        serializer = FileSerializer(video, context={'user': video.creator})

        self.assertNotEqual(
            serializer.get_thumbnail_url(video),
            '/static/img/play.png'
        )

    def test_get_video_file_type(self):
        video = VideoFileFactory.create()

        serializer = FileSerializer(video, context={'user': video.creator})
        self.assertEqual(serializer.get_file_type(video), 'VideoFile')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0005_auto_20150112_1731'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='display_field',
            field=models.ForeignKey(related_name='display_field_of', default=None, to='categories.Field', null=True),
            preserve_default=False,
        ),
    ]



from rest_framework.serializers import ModelSerializer, ValidationError

from geokey.core.serializers import FieldSelectorSerializer
from .models import User, UserGroup


class UserSerializer(FieldSelectorSerializer):
    """
    Serializer for User model.
    """
    class Meta:
        model = User
        fields = ('id', 'email', 'display_name')

    def validate_display_name(self, value):
        """
        Checks if the display name already exists

        Parameter
        ---------
        value : str
            Display name to be examined

        Returns
        -------
        str
            The display name

        Raises
        ------
        ValidationError
            If the display name exists in the data base
        """
        if User.objects.filter(display_name__iexact=value).exists():
            raise ValidationError('Display name already exists')

        return value

    def validate_email(self, value):
        """
        Checks if the email already exists

        Parameter
        ---------
        value : str
            email to be examined

        Returns
        -------
        str
            The email

        Raises
        ------
        ValidationError
            If the email exists in the data base
        """
        if User.objects.filter(email__iexact=value).exists():
            raise ValidationError('Email already exists')

        return value


class UserGroupSerializer(ModelSerializer):
    """
    Serializer for UserGroup model.
    """
    users = UserSerializer(many=True, fields=('id', 'display_name'))

    class Meta:
        model = UserGroup
        depth = 1
        fields = (
            'id', 'name', 'description', 'users', 'can_contribute',
            'can_moderate'
        )



from django.views.generic import CreateView, TemplateView
from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.utils.safestring import mark_safe
from django.template.defaultfilters import slugify
from django.utils.html import strip_tags

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from braces.views import LoginRequiredMixin

from geokey.projects.models import Project
from geokey.core.decorators import (
    handle_exceptions_for_ajax, handle_exceptions_for_admin
)

from .base import STATUS
from .models import (
    Category, Field, TextField, NumericField, LookupField, LookupValue,
    MultipleLookupField, MultipleLookupValue
)
from .forms import CategoryCreateForm, FieldCreateForm
from .serializer import (
    CategorySerializer, FieldSerializer, LookupFieldSerializer
)
from geokey.contributions.models import Observation


# ############################################################################
#
# Administration views
#
# ############################################################################

class CategoryList(LoginRequiredMixin, TemplateView):
    """
    Displays a list of all catgories for a given project.
    """
    template_name = 'categories/category_list.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the project.

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database

        Returns
        -------
        dict
            context; {'project': <geokey.projects.models.Project>}
        """
        user = self.request.user

        context = super(CategoryList, self).get_context_data()
        context['project'] = Project.objects.as_admin(user, project_id)

        return context


class CategoryOverview(LoginRequiredMixin, TemplateView):
    """
    Landing page when a user clicks on a category. Displays a lis of fields
    assigned to the category.
    """
    template_name = 'categories/category_overview.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, category_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the category.

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        dict
            context; {'project': <geokey.categories.models.Category>}
        """
        user = self.request.user
        category = Category.objects.as_admin(
            user, project_id, category_id)

        context = super(CategoryOverview, self).get_context_data()
        context['category'] = category

        return context


class CategoryCreate(LoginRequiredMixin, CreateView):
    """
    Displays the create Category page and creates the Category
    when POST is requested
    """
    form_class = CategoryCreateForm
    template_name = 'categories/category_create.html'

    @handle_exceptions_for_admin
    def get_context_data(self, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the project.

        Returns
        -------
        dict
            context; {'project': <geokey.projects.models.Project>}
        """
        project_id = self.kwargs['project_id']

        context = super(CategoryCreate, self).get_context_data(**kwargs)
        context['project'] = Project.objects.as_admin(
            self.request.user, project_id
        )
        return context

    def form_valid(self, form):
        """
        Is called when the POSTed data is valid and creates the category.

        Parameters
        ----------
        form : geokey.categories.forms.CategoryCreateForm
            Represents the user input
        """
        data = form.cleaned_data

        project_id = self.kwargs['project_id']
        project = Project.objects.as_admin(self.request.user, project_id)

        category = Category.objects.create(
            project=project,
            creator=self.request.user,
            name=strip_tags(data.get('name')),
            description=strip_tags(data.get('description')),
            default_status=data.get('default_status')
        )

        messages.success(self.request, "The category has been created.")
        return redirect(
            'admin:category_overview',
            project_id=project.id,
            category_id=category.id
        )


class CategorySettings(LoginRequiredMixin, TemplateView):
    """
    Displays the category settings page
    """
    template_name = 'categories/category_settings.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, category_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the category.

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        dict
            {
                'category': <geokey.categories.models.Category>
                'admin': Boolean, indicates if the user is admin
                'status_types': List of status types for categoies
                'num_contributions': Number of contributions of that category
            }
        """
        user = self.request.user
        category = Category.objects.as_admin(
            user, project_id, category_id)
        num_contributions = Observation.objects.filter(
            category=category).count()

        return {
            'category': category,
            'admin': category.project.is_admin(user),
            'status_types': STATUS,
            'num_contributions': num_contributions
        }

    def post(self, request, project_id, category_id):
        """
        Handles the POST request and updates the category

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id, category_id)
        category = context.pop('category', None)

        if category is not None:
            data = request.POST

            category.name = strip_tags(data.get('name'))
            category.description = strip_tags(data.get('description'))
            category.default_status = data.get('default_status')

            if category.fields.exists():
                display_field = category.fields.get(
                    pk=data.get('display_field')
                )

                if category.display_field != display_field:
                    category.display_field = display_field
                    for obs in category.observation_set.all():
                        obs.update_display_field()
                        obs.save()

            category.save()

            messages.success(self.request, "The category has been updated.")
            context['category'] = category
        return self.render_to_response(context)


class CategoryDisplay(LoginRequiredMixin, TemplateView):
    """
    Displat the category display settings, i.e. where colour and icon for the
    category can be set.
    """
    template_name = 'categories/category_display.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, category_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the category.

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        dict
            {
                'category': <geokey.categories.models.Category>
            }
        """
        user = self.request.user
        category = Category.objects.as_admin(user, project_id, category_id)

        return super(CategoryDisplay, self).get_context_data(
            category=category, **kwargs)

    def post(self, request, project_id, category_id):
        """
        Handles the POST request and updates the category display settings

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id, category_id)
        category = context.pop('category', None)

        if category is not None:
            data = request.POST
            symbol = request.FILES.get('symbol')
            category.colour = data.get('colour')

            if symbol is not None:
                category.symbol = symbol
            elif data.get('clear-symbol') == 'true':
                category.symbol = None

            category.save()

            messages.success(
                self.request, 'The display settings have been updated')
            context['category'] = category

        return self.render_to_response(context)


class CategoryDelete(LoginRequiredMixin, TemplateView):
    """
    Deletes a category if requesting user is admin of the project
    """
    template_name = 'base.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, category_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the category.

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        dict
            {
                'category': <geokey.categories.models.Category>
            }
        """
        user = self.request.user
        category = Category.objects.as_admin(
            user, project_id, category_id)
        return super(CategoryDelete, self).get_context_data(
            category=category, **kwargs)

    def get(self, request, project_id, category_id):
        """
        Handles the GET request and deletes the category

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to list of categories overview.

        django.http.HttpResponse
            If user is not admin of the project, the error message is rendered.
        """
        context = self.get_context_data(project_id, category_id)
        category = context.pop('category', None)

        if category is not None:
            category.delete()

            messages.success(self.request, "The category has been deleted.")
            return redirect('admin:category_list', project_id=project_id)

        return self.render_to_response(context)


class FieldCreate(LoginRequiredMixin, CreateView):
    """
    Displays the create field page
    """
    form_class = FieldCreateForm
    template_name = 'categories/field_create.html'

    @handle_exceptions_for_admin
    def get_context_data(self, data=None, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the category and available field types

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Returns
        -------
        dict
            {
                'category': <geokey.categories.models.Category>
                'fieldtypes': List of str, representing the field types
            }
        """
        project_id = self.kwargs['project_id']
        category_id = self.kwargs['category_id']

        context = super(FieldCreate, self).get_context_data(**kwargs)

        context['category'] = Category.objects.as_admin(
            self.request.user, project_id, category_id
        )
        context['fieldtypes'] = Field.get_field_types()
        return context

    def form_valid(self, form):
        """
        Is called when the POSTed data is valid and creates the field.

        Parameters
        ----------
        form : geokey.categories.forms.FieldCreateForm
            Represents the user input

        Return
        ------
        Redirects to field setting page of the created field
        """
        project_id = self.kwargs['project_id']
        category_id = self.kwargs['category_id']
        data = form.cleaned_data
        category = Category.objects.as_admin(
            self.request.user, project_id, category_id)

        proposed_key = slugify(strip_tags(data.get('name')))
        if len(proposed_key) < 1:
            proposed_key = 'key'
        suggested_key = proposed_key

        count = 1
        while category.fields.filter(key=suggested_key).exists():
            suggested_key = '%s-%s' % (proposed_key, count)
            count = count + 1

        field = Field.create(
            strip_tags(data.get('name')),
            suggested_key,
            strip_tags(data.get('description')),
            data.get('required'),
            category,
            self.request.POST.get('type')
        )

        if isinstance(field, TextField):
            field.textarea = self.request.POST.get('textarea') or False
            field.maxlength = self.request.POST.get('maxlength') or None

        elif isinstance(field, NumericField):
            field.minval = self.request.POST.get('minval') or None
            field.maxval = self.request.POST.get('maxval') or None

        field.save()

        field_create_url = reverse(
            'admin:category_field_create',
            kwargs={
                'project_id': project_id,
                'category_id': category_id
            }
        )

        messages.success(
            self.request,
            mark_safe('The field has been created. <a href="%s">Add another '
                      'field.</a>' % field_create_url)
        )

        return redirect(
            'admin:category_field_settings',
            project_id=category.project.id,
            category_id=category.id,
            field_id=field.id
        )


class FieldSettings(LoginRequiredMixin, TemplateView):
    """
    Displays the field settings page
    """
    template_name = 'categories/field_settings.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, category_id, field_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the field and available field types

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database
        field_id : int
            Identifier of the field in the database

        Returns
        -------
        dict
            {
                'field': <geokey.categories.models.Field>
                'status_types': List of str, representing the status types
                'is_display_field : Boolean, indicates if field is display
                    field
            }
        """
        user = self.request.user
        field = Field.objects.as_admin(
            user, project_id, category_id, field_id)
        context = super(FieldSettings, self).get_context_data(**kwargs)
        context['field'] = field
        context['status_types'] = STATUS
        context['is_display_field'] = (field == field.category.display_field)

        return context

    def post(self, request, project_id, category_id, field_id):
        """
        Handles the POST request and updates the field

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database
        field_id : int
            Identifier of the field in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(
            project_id,
            category_id,
            field_id
        )
        field = context.pop('field', None)

        if field is not None:
            data = request.POST

            field.name = strip_tags(data.get('name'))
            field.description = strip_tags(data.get('description'))
            field.required = data.get('required') or False

            if isinstance(field, TextField):
                field.textarea = data.get('textarea') or False
                field.maxlength = data.get('maxlength') or None

            elif isinstance(field, NumericField):
                field.minval = data.get('minval') or None
                field.maxval = data.get('maxval') or None

            field.save()

            messages.success(self.request, "The field has been updated.")
            context['field'] = field

        return self.render_to_response(context)


class FieldDelete(LoginRequiredMixin, TemplateView):
    template_name = 'base.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, category_id, field_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the field and available field types

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database
        field_id : int
            Identifier of the field in the database

        Returns
        -------
        dict
            {
                'field': <geokey.categories.models.Field>
            }
        """
        user = self.request.user
        field = Field.objects.as_admin(
            user, project_id, category_id, field_id)
        return super(FieldDelete, self).get_context_data(
            field=field, **kwargs)

    def get(self, request, project_id, category_id, field_id):
        """
        Handles the GET request and deletes the field

        Parameter
        ---------
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database
        field_id : int
            Identifier of the field in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to list of fields overview.

        django.http.HttpResponse
            If user is not admin of the project, the error message is rendered.
        """
        context = self.get_context_data(project_id, category_id, field_id)
        field = context.pop('field', None)

        if field is not None:
            field.delete()

            messages.success(self.request, "The field has been deleted.")
            return redirect(
                'admin:category_overview',
                project_id=project_id,
                category_id=category_id
            )

        return self.render_to_response(context)


# ############################################################################
#
# AJAX API views
#
# ############################################################################

class CategoryUpdate(APIView):
    """
    API Endpoints for a category in the AJAX API.
    /ajax/projects/:project_id/categories/:category_id
    """
    @handle_exceptions_for_ajax
    def get(self, request, project_id, category_id):
        """
        Handles the GET request

        Parameter
        ---------
        request : rest_framework.request.Request
            Object reprensting the request
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Return
        ------
        rest_framework.response.Response
            Reponse to the request
        """
        category = Category.objects.as_admin(
            request.user, project_id, category_id)

        serializer = CategorySerializer(category)
        return Response(serializer.data)

    @handle_exceptions_for_ajax
    def put(self, request, project_id, category_id):
        """
        Handles the POST request and updates the category

        Parameter
        ---------
        request : rest_framework.request.Request
            Object reprensting the request
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Return
        ------
        rest_framework.response.Response
            Reponse to the request
        """

        category = Category.objects.as_admin(
            request.user, project_id, category_id)

        serializer = CategorySerializer(
            category, data=request.DATA, partial=True,
            fields=('id', 'name', 'description', 'status'))

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class FieldUpdate(APIView):
    """
    API endpoints for fields
    /ajax/projects/:project_id/categories/:category_id/fields/
    :field_id
    """
    @handle_exceptions_for_ajax
    def put(self, request, project_id, category_id, field_id,
            format=None):
        """
        Handles the POST request and updates the category

        Parameter
        ---------
        request : rest_framework.request.Request
            Object reprensting the request
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database
        field_id : int
            Identifier of the field in the database

        Return
        ------
        rest_framework.response.Response
            Reponse to the request
        """
        field = Field.objects.as_admin(
            request.user, project_id, category_id, field_id)

        serializer = FieldSerializer(
            field, data=request.DATA, partial=True
        )

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class FieldLookups(APIView):
    """
    API endpoint for lookupvalues
    /ajax/projects/:project_id/categories/:category_id/fields/
    :field_id/lookupvalues
    """
    @handle_exceptions_for_ajax
    def post(self, request, project_id, category_id, field_id,
             format=None):
        """
        Handles the POST request and adds a lookupvalue to the field

        Parameter
        ---------
        request : rest_framework.request.Request
            Object reprensting the request
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database
        field_id : int
            Identifier of the field in the database

        Return
        ------
        rest_framework.response.Response
            Reponse to the request
        """
        field = Field.objects.as_admin(
            request.user, project_id, category_id, field_id)
        name = strip_tags(request.DATA.get('name'))

        if isinstance(field, LookupField):
            LookupValue.objects.create(name=name, field=field)

            serializer = LookupFieldSerializer(field)
            return Response(serializer.data, status=status.HTTP_201_CREATED)

        elif isinstance(field, MultipleLookupField):
            MultipleLookupValue.objects.create(name=name, field=field)

            serializer = LookupFieldSerializer(field)
            return Response(serializer.data, status=status.HTTP_201_CREATED)

        else:
            return Response(
                {'error': 'This field is not a lookup field'},
                status=status.HTTP_404_NOT_FOUND
            )


class FieldLookupsUpdate(APIView):
    """
    API endpoint for lookupvalues
    /ajax/projects/:project_id/categories/:category_id/fields/
    :field_id/lookupvalues/:value_id
    """
    @handle_exceptions_for_ajax
    def delete(self, request, project_id, category_id, field_id,
               value_id):
        """
        Handles the DELETE request and removes the lookupvalue the category

        Parameter
        ---------
        request : rest_framework.request.Request
            Object reprensting the request
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database
        field_id : int
            Identifier of the field in the database
        value_id : int
            Identifier of the lookupvalue in the database

        Return
        ------
        rest_framework.response.Response
            Reponse to the request
        """
        field = Field.objects.as_admin(
            request.user, project_id, category_id, field_id)

        if (isinstance(field, LookupField) or
                isinstance(field, MultipleLookupField)):
            field.lookupvalues.get(pk=value_id).delete()
            return Response(status=status.HTTP_204_NO_CONTENT)
        else:
            return Response(
                {'error': 'This field is not a lookup field'},
                status=status.HTTP_404_NOT_FOUND
            )


class FieldsReorderView(APIView):
    """
    API endpoint to reorder the fields of a category
    /ajax/projects/:project_id/categories/:category_id/fields/re-order/
    """
    @handle_exceptions_for_ajax
    def post(self, request, project_id, category_id):
        """
        Handles the DELETE request and removes the lookupvalue the category

        Parameter
        ---------
        request : rest_framework.request.Request
            Object reprensting the request
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Return
        ------
        rest_framework.response.Response
            Reponse to the request
        """
        category = Category.objects.as_admin(
            request.user, project_id, category_id)
        try:
            category.re_order_fields(request.DATA.get('order'))

            serializer = CategorySerializer(category)
            return Response(serializer.data)
        except Field.DoesNotExist:
            return Response(
                {'error': 'One or more field ids where not found.'},
                status=status.HTTP_400_BAD_REQUEST
            )


# ############################################################################
#
# Public API views
#
# ############################################################################

class SingleCategory(APIView):
    """
    API endpoint for a single category
    /api/projects/:project_id/categories/:category_id
    """
    @handle_exceptions_for_ajax
    def get(self, request, project_id, category_id):
        """
        Handles the GET request and returns the complete category including
        all fields

        Parameter
        ---------
        request : rest_framework.request.Request
            Object reprensting the request
        project_id : int
            Identifier of the project in the database
        category_id : int
            Identifier of the category in the database

        Return
        ------
        rest_framework.response.Response
            Reponse to the request
        """
        category = Category.objects.get_single(
            request.user, project_id, category_id)

        serializer = CategorySerializer(category)
        return Response(serializer.data)
